/***********************************************************
 * $Id: ComparableComparator.java 20 2007-03-01 10:45:50Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: Feb 28, 2007
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.comparators;

import java.util.Comparator;

/**
 * This class is a comparator, which compares {@link Comparable} instances
 * by forwarding the {@link Comparable#compareTo(Object)} method to the
 * implemented {@link #compareTo(Comparable)} method.  
 * 
 * @author wglas
 */
public class ComparableComparator implements Comparator
{
    private static Comparator INSTANCE = new ComparableComparator();
    
    /**
     * This is the preferred methos to acquire an instance of ComparableComparator. 
     * 
     * @return A static instance of ComparableComparator.
     */
    public static Comparator getInstance()
    {
        return INSTANCE;
    }
    
    /* (non-Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object a, Object b) {
       
        if (a == null)
            return b==null ? 0 : -1;
        
        if (!(a instanceof Comparable))
            throw new ClassCastException("First argument of class ["+a.getClass()+"] does not implement Comparable");
        
        if (b==null) return 1;
        
        return ((Comparable)a).compareTo(b);
    }

}
