/***********************************************************
 * $Id: DefaultComparatorFactory.java 22 2007-03-01 16:19:43Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: Feb 28, 2007
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.comparators;

import java.util.Comparator;

/**
 * This default implementation of ComparatorFactory simply returns
 * a ComparableComparator for Comparables.
 * 
 * @author wglas
 */
public class DefaultComparatorFactory implements ComparatorFactory {

    private static DefaultComparatorFactory INSTANCE = new DefaultComparatorFactory();
    
    /**
     * @return A static DefaultComparatorFactory instance.
     */
    public static DefaultComparatorFactory getInstance()
    {
        return INSTANCE;
    }
    
    /* (non-Javadoc)
     * @see org.clazzes.util.comparators.ComparatorFactory#getComparator(java.lang.Class)
     */
    public Comparator getComparator(Class clazz) {
        
        if (clazz.isPrimitive())
        {
            if (clazz==Void.TYPE)
                throw new ClassCastException("Cannot create a comparator for void objects.");
        }
        else if (!Comparable.class.isAssignableFrom(clazz))
            throw new ClassCastException("The supplied class ["+clazz+"] does not implement Comparable.");
            
        return ComparableComparator.getInstance();
    }

}
