/***********************************************************
 * $Id: Pair.java 14 2007-02-09 13:13:12Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-12-27
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.lang;

/**
 * @author wglas
 * 
 * A pair of two java objects, implemented as a Java 1.5 generic. The lack of
 * this type is actually a bug of JRE 1.5
 */
public class Pair<A, B>
{

	A first;

	B second;

	/**
	 * Contructs a pair holding two null values.
	 */
	public Pair()
	{
		super();
		this.first = null;
		this.second = null;
	}

	/**
	 * Contructs a pair holding the given objects.
	 */
	public Pair(A a, B b)
	{
		super();
		this.first = a;
		this.second = b;
	}

	/**
	 * Contructs a pair holding the objects of the given pair.
	 */
	public Pair(Pair<A, B> p)
	{
		super();
		this.first = p.getFirst();
		this.second = p.getSecond();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
    public boolean equals(Object o)
	{
		if (!(o instanceof Pair))
			return false;

		Pair p = (Pair) o;

        if (this.first == null) {
            if (p.getFirst() != null)
                return false;
        } else {
            if (!this.first.equals(p.getFirst()))
                return false;
        }
        if (this.second == null) {
            return p.getSecond() == null;
        } else {
            return this.second.equals(p.getSecond());
        }
    }
    
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
    public int hashCode()
	{
		// obey the following rules:
		// Pair(null,null).hashCode() = -1;
		// Pair(a,null).hashCode() = a.hashCode();
		// Pair(null,b).hashCode() = -1 ^ b.hashCode();
		// Pair(a,b).hashCode() = a.hashCode() ^ b.hashCode();
		int ret = -1;
		if (this.first != null)
			ret = this.first.hashCode();
		if (this.second != null)
			ret ^= this.second.hashCode();
		return ret;
	}

	/**
	 * @return Returns the first object.
	 */
	public A getFirst()
	{
		return this.first;
	}

	/**
	 * @param first
	 *            The first object to set.
	 */
	public void setFirst(A first)
	{
		this.first = first;
	}

	/**
	 * @return Returns the second object.
	 */
	public B getSecond()
	{
		return this.second;
	}

	/**
	 * @param second
	 *            The second objet to set.
	 */
	public void setSecond(B second)
	{
		this.second = second;
	}

}
