/***********************************************************
 * $Id: FieldAccessor.java 4 2006-12-26 19:16:57Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-03-13
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/


package org.clazzes.util.sec;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/** 
 * A Tool to hash passwords and test the PasswordHasher classes
 */
public class Hasher
{

    protected static Log log=LogFactory.getLog(Hasher.class);
    
    protected static PasswordHasherFactory passwordHasherFactory;

	/**
	 * @param args
	 */
	@SuppressWarnings("null")
    public static void main(String[] args) throws java.lang.Exception
	{
		// if called as "program", we can perhaps do different things
		// make a simple stdout log
		/*
     	log = Logger.getLogger("org.clazzes.util.sec.Hasher");
		BasicConfigurator.configure();
		log.addAppender(new ConsoleAppender());
		log.setLevel(Level.DEBUG);
		*/
        // parse eventual params
		// what to do
        passwordHasherFactory = new HasherFactory();
        String command = System.getenv("CMD");
		if ((command == null) || (command.length() == 0))
		{
			command = "hash";
			log.info("Using default CMD hash");
		} else
		{
			log.info("Using given CMD " + command);
		}
		// password
		String pwcleartext = System.getenv("PW");
		if ((pwcleartext == null) || (pwcleartext.length() == 0))
		{
			log.fatal("No PW given.");
			System.exit(2);
		} else
		{
			log.info("Using given PW " + pwcleartext);
		}
		// work ...
		if (command.equals("hash"))
		{
			// salt?
			String salt = System.getenv("SALT");
			if ((salt == null) || (salt.length() == 0))
			{
				salt = HashTools.randomSalt();
				log.info("No SALT given, using random salt " + salt);
			} else
			{
				log.info("Using given SALT " + salt);
			}
			// alg
			String algo_name = System.getenv("ALG");
			if ((algo_name == null) || (algo_name.length() == 0))
			{
				algo_name = "SSHA1"; // SSHA1
				log.info("No ALG given, using default algorithm "
						+ algo_name);
			} else
			{
				log.info("Using given ALG " + algo_name);
			}
			// hasher
			PasswordHasher pwhasher = passwordHasherFactory.getPasswordHasher(algo_name);
			if (pwhasher == null)
			{
				log
						.fatal("Could not find hasher for algorithm "
								+ algo_name);
				System.exit(3);
			} else
			{
				log.info("Got PasswordHasher of class "
						+ pwhasher.getClass().getName());
			}
			String hashed = pwhasher.hashPassword(pwcleartext, salt);
			log.info("Computed password to " + hashed);
			System.exit(0);
		}
		if (command.equals("check"))
		{
			// hashed?
			String hashed = System.getenv("HASHED");
			if ((hashed == null) || (hashed.length() == 0))
			{
				log.fatal("No HASHED pw given to check");
				System.exit(4);
			} else
			{
				log.info("Using HASHED " + hashed);
			}
			String algo_name = HashTools.parseAlorithmName(hashed);
			log.info("Parsed algo_name to " + algo_name);
			PasswordHasher pwhasher = passwordHasherFactory.getPasswordHasher(algo_name);
			if (pwhasher == null)
			{
				log
						.fatal("Could not find hasher for algorithm "
								+ algo_name);
				System.exit(5);
			} else
			{
				log.info("Got PasswordHasher of class "
						+ pwhasher.getClass().getName());
			}
			boolean pwok = pwhasher.checkPassword(pwcleartext, hashed);
			log.info("The PW seems to be " + (pwok ? "OK" : "WRONG"));
			System.exit(0);
		}
		log.fatal("What did you want? " + command + "?");
		System.exit(1);
	}

 
}
