/***********************************************************
 * $Id: ThreadTimer.java 12 2007-01-18 10:28:04Z wglas $
 * 
 * Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 2006-12-27
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 * 
 ***********************************************************/

package org.clazzes.util.thread;

import java.util.TimerTask;

/**
 * @author andreas
 * 
 * Helper class for a timed thread execution. This class is called by the
 * {@link java.util.Timer} It invokes {@link #run()}, which starts the Thread.
 */
public class ThreadTimer extends TimerTask
{

	/**
	 * Thread to control
	 */
	private TimedThread _thread;

	/**
	 * if the thread is started this boolean is true
	 */
	private boolean started;

	/**
	 * composition to use the threadstopper
	 */
	private ThreadStopper stopper;

	/**
	 * contructes the object and sets the thread to run
	 * 
	 * @param thread
	 */
	public ThreadTimer(TimedThread thread)
	{
		this._thread = thread;
		this.started = false;
		this.stopper = new ThreadStopper();
		thread.setThreadTimer(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.TimerTask#run()
	 */
	@Override
	public void run()
	{
		if (!this.started)
		{
			((Thread) this._thread).start();
			this.started = true;
		} else
			this.stopper.resume();
	}

	/**
	 * 
	 */
	public void requestStop()
	{
		this.stopper.requestStop();

	}

	/**
	 * @return
	 */
	public boolean isToBeFinished()
	{
		return this.stopper.isToBeFinished();
	}

	/**
	 * @throws InterruptedException
	 * 
	 */
	public void waitToBeFinished() throws InterruptedException
	{
		this.stopper.waitToBeFinished();
	}

}
